import matplotlib.pyplot as plt
import numpy as np
import pygimli as pg
import pybert as pb


class DebyeDecompositionTD(pg.core.ModellingBase):
    """Debye decomposition modelling class."""
    def __init__(self, tvec, tauvec):
        """Init."""
        self.t = tvec
        self.tau = tauvec
        self.mesh = pg.core.createMesh1D(len(tauvec))
        super().__init__(self.mesh)
        self.A = pg.Matrix()
        self.A.resize(len(self.t), len(self.tau))
        for i, tau in enumerate(self.tau):
            self.A.setCol(i, pg.exp(-self.t/tau))

        self.setJacobian(self.A)

    def response(self, model):
        """Forward response."""
        return self.A * model

    def createJacobian(self, model):
        pass


def fitDecay(decayObs, tauvec, tvec):
    """.Fit decay for a give tau vector."""
    f = DebyeDecompositionTD(tvec, tauvec)
    model = pg.Vector(len(spec.tau), 0.0001)
    transLog = pg.core.TransLog()
    f.regionManager().setConstraintType(10)
    inv = pg.core.RInversion(decayObs, f)
    inv.setTransModel(transLog)
    inv.setAbsoluteError(0.001)
    inv.setLambda(0.1)
    # inv.setLambdaFactor(0.9)
    inv.setModel(model)
    inv.setReferenceModel(model)
    # inv.setBlockyModel(True)
    model = inv.run()
    return model.array(), inv.response().array()



fdip = pb.FDIPdata('../IP1/SIP256C/IP1_SIP256C')
fdip.filter(fmax=100)
tdip = pb.TDIPdata('../IP1/Terrameter/IP1_Terrameter.dip')
tdip.filter(tmin=10e-3)
# %%
fig, ax = plt.subplots()
col1 = ['C9', 'C1', 'C6']  # light blue, orange, pink
col2 = ['C0', 'C3', 'C4']  # dark blue, red, magenta
# %%
fig, ax = plt.subplots()
a = 26
start = [4, 9, 13]  # 4s proc
for i, n in enumerate([1, 4, 7]):
    abmn = [a, a+1, a+1+n, a+n+2]
    st = (' {:d}'*2).format(*abmn[2:])
    # FD
    spec = fdip.getDataSpectrum(abmn=abmn)
    spec.fitDebyeModel(mint=1e-3, maxt=100, nt=100, cType=1, lam=300, new=True, showFit=False)
    decay = tdip.getDataDecay(abmn) / 1000  # mV/V !!!!
    i0 = np.argmax(np.diff(decay) < 0)
    i0 = start[i]
    model, resp = fitDecay(decay[i0:], spec.tau, tdip.t[i0:])
    ax.loglog(spec.tau, spec.mDD*1000, color=col2[i],  # FD
              label='SIP256C'+st)
    ax.loglog(spec.tau, model*1000, color=col1[i], ls='--',  # TD
              label='Terrameter'+st)

ax.legend()  # loc='lower left')
ax.grid(True)
ax.set_xlim(1e-2, 4)
ax.set_xlabel(r'Debye relaxation time $\tau$ [s]')
ax.set_ylabel('chargeability [mV/V]')
fig.savefig('figure09.pdf', bbox_inches='tight')
